using System;
using System.Linq;
using System.Windows.Media;
using System.Windows.Threading;
using Telerik.Windows.Controls;
using Telerik.Windows.Data;

namespace WPFApp
{
    public class ViewModel : ViewModelBase
    {
        public RadObservableCollection<PlotInfo> Data { get; private set; }
        public RadObservableCollection<PlotInfo> PlaceholderData { get; private set; }

        private DateTime firstVisibleDate;
        public DateTime FirstVisibleDate
        {
            get
            {
                return this.firstVisibleDate;
            }
            set
            {
                if (this.firstVisibleDate == value)
                    return;
                this.firstVisibleDate = value;
                OnPropertyChanged("FirstVisibleDate");
            }
        }

        private DateTime lastDate;
        public DateTime LastDate
        {
            get
            {
                return this.lastDate;
            }
            set
            {
                if (this.lastDate == value)
                    return;
                this.lastDate = value;
                OnPropertyChanged("LastDate");
            }
        }

        private double delay = 500.0;
        public double Delay
        {
            get { return this.delay; }
            set 
            {
                if (this.delay == value)
                    return;

                this.delay = value; 
                this.timer.Interval = TimeSpan.FromMilliseconds(value);
                this.OnPropertyChanged("Delay");
            }
        }

        private double axisRange = 10.0;
        public double AxisRange
        {
            get { return this.axisRange; }
            set
            {
                if (this.axisRange == value)
                    return;

                this.axisRange = value;
                this.OnPropertyChanged("AxisRange");
            }
        }

        private readonly DispatcherTimer timer;

        public bool ShouldAddItems = false;

        public ViewModel()
        {
            this.Data = new RadObservableCollection<PlotInfo>();
            this.PlaceholderData = new RadObservableCollection<PlotInfo>();

            this.LastDate = DateTime.Now;
            this.FirstVisibleDate = this.LastDate.AddSeconds(-1.0 * axisRange);

            this.timer = new DispatcherTimer();
            this.timer.Interval = TimeSpan.FromMilliseconds(delay);
            this.timer.Tick += this.OnTimer;
        }

        public ViewModel(double milisecondsDelay, double axisRangeInSeconds)
        {
            this.Data = new RadObservableCollection<PlotInfo>();
            this.PlaceholderData = new RadObservableCollection<PlotInfo>();

            this.delay = milisecondsDelay;
            this.axisRange = axisRangeInSeconds;

            this.LastDate = DateTime.Today;
            this.FirstVisibleDate = this.LastDate.AddSeconds(-1.0 * axisRange);

            this.timer = new DispatcherTimer();
            this.timer.Interval = TimeSpan.FromMilliseconds(delay);
            this.timer.Tick += this.OnTimer;
        }

        public void StartTimer()
        {
            this.timer.Start();
        }

        public void StopTimer()
        {
            this.timer.Stop();
        }

        private void OnTimer(object sender, EventArgs e)
        {
            this.LastDate = this.LastDate.AddMilliseconds(delay);
            this.FirstVisibleDate = this.LastDate.AddSeconds(-1.0 * axisRange);

            RefreshPlaceHolderData();

            if (ShouldAddItems)
                this.AddDataItem(this.LastDate);
        }

        //Keeps the placeholder data within the axis range (beween FirstVisibleDate and LastDate
        private void RefreshPlaceHolderData()
        {
            this.PlaceholderData.SuspendNotifications();
            this.PlaceholderData.Clear();

            long newAverage = (this.LastDate.Ticks + this.FirstVisibleDate.Ticks) / 2;

            this.PlaceholderData.Add(new PlotInfo { Date = new DateTime(newAverage), YVal = 50 });
            this.PlaceholderData.Add(new PlotInfo { Date = new DateTime(newAverage).AddSeconds(1), YVal = 50 });

            this.PlaceholderData.ResumeNotifications();
        }

        public void AddDataItem(DateTime timeStamp)
        {
            this.Data.SuspendNotifications();

            this.Data.Add(new PlotInfo { Date = timeStamp, YVal = rnd.Next(5, 100) });

            //Remove data outside of the visible range
            if (this.Data[0].Date < this.firstVisibleDate)
            {
                this.Data.RemoveAt(0);
            }

            this.Data.ResumeNotifications();
        }

        private static readonly Random rnd = new Random();
    }
}